<?php

namespace Drupal\media_folder_management;

use Drupal\Component\Utility\SortArray;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Link;
use Drupal\Core\Render\Markup;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\media_folder_management\Entity\Access\MediaFolderFileAccessCheck;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Session\AccountInterface;

/**
 * Service which builds the file explorer widget.
 */
class MediaFoldersUiBuilder {

  use StringTranslationTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The currently active request object.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * Current user account.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, RequestStack $request_stack, AccountInterface $current_user) {
    $this->entityTypeManager = $entity_type_manager;
    $this->request = $request_stack->getCurrentRequest();
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public static function dialogOptions() {
    return [
      'classes' => [
        'ui-dialog' => 'media-folders-widget-modal',
      ],
      'title' => t('Add or select files'),
      'height' => '75%',
      'width' => '75%',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildUi($state = NULL) {
    if (!$state) {
      $state = MediaFoldersState::fromRequest($this->request);
    }

    $folder_id = 0;
    $folders = [];
    $tree = $this->entityTypeManager->getStorage('taxonomy_term')->loadTree('media_folder', $folder_id, 1, TRUE);
    $view_type = 'thumbs';

    foreach ($tree as $term) {
      $classes = [
        'folder-icon',
        'folder-icon-folder-empty',
      ];

      if (!MediaFolderFileAccessCheck::hasFolderPermission('view', $term, $this->currentUser)) {
        continue;
      }
      $wrapper_classes = ['folder-folder'];
      if (!MediaFolderFileAccessCheck::hasFolderPermission('create', $term, $this->currentUser)) {
        $wrapper_classes[] = 'not-droppable';
      }

      $folders[] = [
        '#theme' => 'media_folder_folder',
        '#title' => $term->getName(),
        '#weight' => $term->changed->value,
        '#desc' => (!empty($term->getDescription())) ? Markup::create(strip_tags($term->getDescription())) : '',
        '#link' => Url::fromRoute('media_folder_management.folder', ['folder' => $term->id()]),
        '#classes' => implode(' ', $classes),
        '#wrapper_classes' => implode(' ', $wrapper_classes),
        '#attr' => '',
        '#size' => '',
        '#ftype' => '',
        '#actions' => '',
        '#fid' => $term->id(),
        '#cache' => [
          'tags' => [
            'media_folder_item:' . $term->id(),
          ],
          'contexts' => [
            'user',
          ],
        ],
      ];
    }
    $folders_ordered = $this->itemsOrder($folders);
    $navbar = $this->geAllUserFoldersTree($folder_id);
    $subfolders_class = 'not-droppable';

    $explorer = [
      '#theme' => 'media_folder_widget',
      '#folders' => [
        '#theme' => 'media_folder_folders',
        '#folders' => $folders_ordered,
      ],
      '#navbar' => [
        '#theme' => 'media_folder_navbar',
        '#folders' => $navbar,
      ],
      '#class' => $view_type,
      '#subfolders_class' => $subfolders_class,
      '#fid' => $folder_id,
      '#cache' => [
        'tags' => [
          'media_folder_page:' . $folder_id,
        ],
        'contexts' => [
          'user',
        ],
      ],
      '#attached' => [
        'drupalSettings' => [
          'media_folders' => [
            'selection_remaining' => $state->getAvailableSlots(),
            'state_parameters' => $state->getOpenerParameters(),
          ],
        ],
      ],
    ];

    return $explorer;
  }

  /**
   * {@inheritdoc}
   */
  private function itemsOrder($items) {
    uasort($items, [SortArray::class, 'sortByTitleProperty']);

    return $items;
  }

  /**
   * {@inheritdoc}
   */
  public function treeNestedArray(&$build, $tree, $my_folders, $folder_id) {
    foreach ($tree as $term) {
      if (!in_array($term->id(), $my_folders)) {
        continue;
      }
      $key = strtolower($term->getName()) . '-' . $term->id();
      $link = Link::fromTextAndUrl($term->getName(), Url::fromRoute('media_folder_management.folder', ['folder' => $term->id()]));
      $build[$key] = [
        'id' => $term->id(),
        'link' => $link,
        'class' => ($folder_id == $term->id()) ? 'active' : '',
        'children' => [],
      ];
      $children = $this->entityTypeManager->getStorage('taxonomy_term')->getChildren($term);
      if (!empty($children)) {
        $this->treeNestedArray($build[$key]['children'], $children, $my_folders, $folder_id);
      }
    }
    ksort($build);
  }

  /**
   * {@inheritdoc}
   */
  public function treeNestedArrayActiveTrail(&$build, $folder_id, $paths = FALSE) {
    if ($paths === FALSE) {
      $paths = $this->treeNestedArrayActiveTrailPaths($build, $folder_id);
    }

    if (!empty($paths)) {
      foreach ($paths as $path) {
        if (!empty($build[$path])) {
          $build[$path]['class'] .= ' active-trail';
          $this->treeNestedArrayActiveTrail($build[$path]['children'], $folder_id, $paths);
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function treeNestedArrayActiveTrailPaths($build, $folder_id) {
    $paths = [];
    $iterator = new \RecursiveIteratorIterator(new \RecursiveArrayIterator($build), \RecursiveIteratorIterator::SELF_FIRST);
    foreach ($iterator as $v) {
      if (is_array($v) && !empty($v['id']) && $v['id'] == $folder_id) {
        for ($p = [], $i = 0, $z = $iterator->getDepth(); $i <= $z; $i++) {
          $p = $iterator->getSubIterator($i)->key();
          if ($p != 'children') {
            $paths[] = $p;
          }
        }
      }
    }

    return $paths;
  }

  /**
   * {@inheritdoc}
   */
  public function geAllUserFoldersTree($folder_id) {
    $folders = $this->entityTypeManager->getStorage('taxonomy_term')->loadTree('media_folder', 0, 1, TRUE);
    $my_folders = MediaFolderFileAccessCheck::getUserViewFoldersAccess(0, $this->currentUser, $this->entityTypeManager->getStorage('taxonomy_term'));
    $build = [];
    $this->treeNestedArray($build, $folders, $my_folders, $folder_id);
    $this->treeNestedArrayActiveTrail($build, $folder_id);

    return $build;
  }

}
